"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Output = void 0;
const tslib_1 = require("tslib");
const fs = tslib_1.__importStar(require("fs"));
const path = tslib_1.__importStar(require("path"));
const string_hash_1 = tslib_1.__importDefault(require("string-hash"));
const find_cache_dir_1 = tslib_1.__importDefault(require("find-cache-dir"));
const taddy_1 = require("taddy");
const DEFAULT_CACHE_DIR = path.join(__dirname, '../cache');
function getCacheDir() {
    return ((0, find_cache_dir_1.default)({ name: 'taddy', create: true }) || DEFAULT_CACHE_DIR);
}
const LAST_INDEX = 0;
const STYLES = [];
function getStylesState() {
    const { rules } = taddy_1.$css.ruleInjector.styleSheet;
    const newRules = rules.slice(LAST_INDEX);
    // there is also "appending" rules, that don't change the index, but change the rule
    // TODO: think how to handle that
    // LAST_INDEX = rules.length;
    const added = [...newRules].map((rule) => rule.cssText || '');
    STYLES.push(...added);
    return {
        added,
    };
}
function getMtime(filepath) {
    return Number(fs.statSync?.(filepath).mtime);
}
const filesMap = new Map();
const contentMap = new Map();
function readFileSync(filepath) {
    if (!fs.existsSync(filepath))
        return '';
    let data = '';
    try {
        const mtime = getMtime(filepath);
        if (filesMap.has(mtime)) {
            return filesMap.get(mtime);
        }
        data = fs.readFileSync(filepath).toString();
        if (!data)
            return '';
        filesMap.set(mtime, data);
        contentMap.set(data, mtime);
    }
    catch (e) {
        console.error('error', e);
    }
    return data;
}
function mkdir(filepath) {
    fs.mkdirSync(path.dirname(filepath), { recursive: true });
}
function appendFile(filepath, append) {
    // const code = content + append;
    // if (contentMap.has(code)) {
    //     return;
    // }
    // fs.appendFile(filepath, append, () => {});
    mkdir(filepath);
    fs.appendFileSync(filepath, append);
}
function writeFile(filepath, code) {
    // const code = content + append;
    // if (contentMap.has(code)) {
    //     return;
    // }
    // fs.appendFile(filepath, append, () => {});
    mkdir(filepath);
    fs.writeFileSync(filepath, code);
}
function cleanFileCache(filepath) {
    fs.rmdirSync(filepath, { recursive: true });
}
/** don't merge declarations in plugin, currently considering only dev mode */
taddy_1.$css.ruleInjector.styleSheet.options.mergeDeclarations = false;
class Output {
    env;
    config;
    filepath;
    defaultCacheDir = getCacheDir();
    constructor({ env, config }) {
        this.env = env;
        this.config = Object.assign({
            // getCSSFilename = (content) => `atoms-${stringHash(content)}.css`,
            cacheDir: this.defaultCacheDir,
        }, config);
        // const filename = this.resolveFilepath(this.config.cssFilename);
        // this.filepath = this.resolveFilepath(this.config.cssFilepath, filename);
        this.filepath = path.join(this.config.cacheDir, 'taddy.css');
        mkdir(this.filepath);
        // console.log('filepath', this.filepath);
        /* clean cache */
        // fs.writeFileSync(this.filepath, '');
    }
    resolveFilepath(getter, ...params) {
        if (typeof getter !== 'string') {
            // @ts-expect-error TODO: fix getter params
            return getter(...params);
        }
        return getter;
    }
    save({ sourceMap, filenameRelative, target, }) {
        const stylesData = readFileSync(this.filepath);
        const { added } = getStylesState();
        // console.log({added});
        // TODO: improve filter performance
        const diffStyles = added
            .filter((x) => !stylesData.includes(x))
            .join('');
        appendFile(this.filepath, diffStyles);
        // appendFile(this.filepath, sourceMap);
        const filenameHash = (0, string_hash_1.default)(`${filenameRelative}`).toString(36);
        const localFileCachePath = path.join(this.config.cacheDir, filenameHash + '.taddy.css');
        const localFileCacheData = readFileSync(localFileCachePath);
        const localFileCacheDiff = added
            .filter((x) => !localFileCacheData.includes(x))
            .join('');
        appendFile(localFileCachePath, localFileCacheDiff);
        const localFileContent = localFileCacheData + localFileCacheDiff;
        const localFileContentHash = (0, string_hash_1.default)(localFileContent).toString(36);
        const localFilename = path.join(filenameHash, localFileContentHash + '.taddy');
        const localFilepath = path.join(this.config.cacheDir, 
        // stringHash(`${filename}:${added}`) + '.taddy.css',
        localFilename);
        writeFile(localFilepath + '.css', localFileContent.replace(/}$/, sourceMap + '}'));
        const importBase = this.config.cacheDir === this.defaultCacheDir
            ? '.cache/taddy'
            : path.relative(path.dirname(filenameRelative), this.config.cacheDir);
        if (target === 'remix') {
            writeFile(localFilepath + '.cjs', `require('./${localFilename}.css');`);
            writeFile(localFilepath + '.js', `import './${localFilename}.css'`);
            return { importName: path.join(importBase, localFilename) };
        }
        return { importName: path.join(importBase, `${localFilename}.css`) };
    }
}
exports.Output = Output;
//# sourceMappingURL=Output.js.map