import * as t from '@babel/types';
import type { PluginPass } from '@babel/core';
import type { NodePath } from '@babel/traverse';
import { SourceMapGenerator } from 'source-map';
import type { Env, Target } from './types';
import { TSProcessor, type TSProcessorOptions } from './TSProcessor';
type ObjectProperties = t.ObjectExpression['properties'];
export type ProcessorConfig = {
    typescript?: boolean | TSProcessorOptions;
    evaluate?: boolean;
    CSSVariableFallback?: boolean;
    optimizeBindings?: boolean;
    target: Target;
};
export type ProcessOptions = {
    env: Env;
    mixin: boolean;
    filename: string;
    filenameRelative: string;
    code: string;
    addImport(name: string): t.ImportSpecifier['local'];
    sourceMap?: string;
    state?: PluginPass;
    sourceMapGenerator?: SourceMapGenerator;
};
type $CSSOptions = {
    postfix?: string;
    at?: {
        name: string;
        query?: string;
    };
};
type CommonOptions = $CSSOptions & {
    properties?: ObjectProperties;
};
type ObjectOptions = CommonOptions & {
    properties: ObjectProperties;
};
export declare class Processor {
    config: ProcessorConfig;
    options: ProcessOptions;
    isStatic: boolean;
    classNameNodes: WeakSet<t.ObjectProperty>;
    optimizationPaths: Set<NodePath<any>>;
    variables: t.ObjectProperty[];
    private prepare;
    constructor({ config }: {
        config: ProcessorConfig;
    });
    get tsProject(): TSProcessor;
    $css(rule: any, options?: $CSSOptions): import("taddy/$css").InternalTaddyStyle;
    evaluate(currentPath: NodePath<any>): {
        value: any;
    } | {
        error: Error;
    };
    isClassNameNode(node: any): boolean;
    classNamesToNode(classNames: object): t.ObjectProperty[];
    stylesToNode(key: string, value: unknown, { postfix, at }?: $CSSOptions): t.ObjectProperty[];
    processAtRule(key: string, path: NodePath<t.ObjectProperty>, { postfix, properties }: Omit<ObjectOptions, 'at'>): void;
    processPropertyValue(key: string, path: NodePath<t.ObjectProperty>, { postfix, at, properties }: ObjectOptions): void;
    processObjectProperty(path: NodePath<t.ObjectProperty>, { postfix, at, properties }: ObjectOptions): void;
    processSpreadElement(path: NodePath<t.SpreadElement>, { postfix, at, properties }: ObjectOptions): void;
    processLogicalExpression(path: NodePath<t.LogicalExpression>, options: CommonOptions): void;
    processObjectExpression(path: NodePath<t.ObjectExpression>, { postfix, at, properties }: CommonOptions): void;
    processConditionalExpression(path: NodePath<t.ConditionalExpression>, options?: CommonOptions): void;
    process(path: NodePath<any>, options?: CommonOptions): void;
    run(callPath: NodePath<t.CallExpression>, options: ProcessOptions): {
        isStatic: any;
        optimizationPaths: Set<NodePath<any>>;
    };
}
export {};
