"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHandlers = void 0;
const tslib_1 = require("tslib");
const t = tslib_1.__importStar(require("@babel/types"));
const string_hash_1 = tslib_1.__importDefault(require("string-hash"));
const taddy_1 = require("taddy");
const Processor_1 = require("./Processor");
const BindingOptimizer_1 = require("./helpers/BindingOptimizer");
function isPathRemoved(path) {
    do {
        if (path?.removed)
            return true;
    } while ((path = path?.parentPath));
    return false;
}
const createHandlers = (config, options) => {
    const mixinsQueue = [];
    const proceedPaths = [];
    const cssQueue = [];
    const processor = new Processor_1.Processor({ config });
    const optimizer = new BindingOptimizer_1.BindingOptimizer();
    let counter = 0;
    const addHashId = (path) => {
        path.node.arguments.push(t.stringLiteral('__' +
            (0, string_hash_1.default)(options.filenameRelative + ':' + String(++counter)).toString(32)));
    };
    const handlers = {
        css: (path, { mixin = false } = {}) => {
            if (path.isMemberExpression() &&
                'name' in path.node.property &&
                path.node.property.name === 'mixin') {
                mixinsQueue.push(path.parentPath);
                return;
            }
            if (path.isCallExpression()) {
                cssQueue.push({ path, mixin });
                return;
            }
        },
        mixin: (path) => {
            mixinsQueue.push(path);
        },
    };
    function processCSS({ path, mixin, }) {
        const result = processor.run(path, {
            ...options,
            mixin,
        });
        if (!result)
            return;
        const { isStatic, optimizationPaths } = result;
        const cssLength = taddy_1.$css.ruleInjector.styleSheet.rules.join('').length;
        // console.log(options.state.file.opts.generatorOpts.sourceFileName);
        const source = options.state?.file.opts.generatorOpts?.sourceFileName;
        const locStart = path.node.loc?.start;
        if (source && locStart) {
            options.sourceMapGenerator?.addMapping({
                generated: {
                    line: 1,
                    column: cssLength,
                },
                source,
                original: locStart,
            });
        }
        proceedPaths.push({
            path,
            isStatic,
            optimizationPaths,
        });
        if (!mixin) {
            addHashId(path);
        }
    }
    return {
        handlers,
        finish() {
            cssQueue.forEach(processCSS);
            let isStatic = true;
            for (const x of proceedPaths) {
                isStatic = isStatic && x.isStatic;
                if (!config.optimizeBindings)
                    continue;
                for (const path of x.optimizationPaths) {
                    optimizer.process(path);
                }
            }
            for (const path of mixinsQueue) {
                if (config.optimizeBindings && isPathRemoved(path)) {
                    // the path binding should be already optimized
                    continue;
                }
                isStatic = false;
                processCSS({ path, mixin: true });
            }
            /**
             * If it's not a static file, map only static usages
             */
            if (!isStatic) {
                for (const x of proceedPaths) {
                    if (!x.isStatic)
                        continue;
                    const { path } = x;
                    path.replaceWith(t.callExpression(t.memberExpression(path.node.callee, t.identifier('static')), path.node.arguments));
                }
            }
            return { isStatic };
        },
    };
};
exports.createHandlers = createHandlers;
//# sourceMappingURL=handlers.js.map