"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BindingOptimizer = void 0;
const tslib_1 = require("tslib");
const t = tslib_1.__importStar(require("@babel/types"));
const findBindings_1 = require("./findBindings");
function isTaddy(binding) {
    const { path } = binding;
    if (!path.isImportSpecifier())
        return false;
    const { imported } = path.node;
    return (t.isIdentifier(imported) &&
        ((path.parentPath.node.source
            .value === 'taddy' &&
            imported.name === 'css') ||
            imported.name === 'mixin'));
}
class BindingOptimizer {
    cache = new Map();
    process(referentPath) {
        let bindings;
        try {
            bindings = (0, findBindings_1.findBindings)(referentPath);
        }
        catch (error) {
            return;
        }
        for (const [binding, paths] of bindings) {
            if (isTaddy(binding))
                continue;
            if (binding.path.removed)
                continue;
            this.process(binding.path);
            (0, findBindings_1.addBinding)(this.cache, binding, paths);
            if (this.cache.get(binding).size < binding.references) {
                continue;
            }
            binding.path.remove();
            const { parentPath } = binding.path;
            const isImportToRemove = parentPath?.isImportDeclaration() &&
                parentPath.node.specifiers.length === 0;
            // // keep imports for development build
            // if (env === 'development') {
            //     return;
            // }
            if (!isImportToRemove) {
                continue;
            }
            parentPath.remove();
        }
    }
}
exports.BindingOptimizer = BindingOptimizer;
//# sourceMappingURL=BindingOptimizer.js.map