"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.evaluate = void 0;
const tslib_1 = require("tslib");
const babel_helper_evaluate_path_1 = tslib_1.__importDefault(require("babel-helper-evaluate-path"));
const sync_rpc_1 = tslib_1.__importDefault(require("sync-rpc"));
const config_1 = require("../config");
const findBindings_1 = require("./findBindings");
const buildCodeByPath_1 = require("./buildCodeByPath");
const evaluateChunk = (0, sync_rpc_1.default)(__dirname + '/evaluate.worker.cjs', 'Evaluate');
const macroRe = new RegExp(config_1.MACRO_NAME.replace('.', '\\.'), 'g');
function evaluate(currentPath, { exec = true }) {
    const result = (0, babel_helper_evaluate_path_1.default)(currentPath);
    if (result.confident) {
        (0, findBindings_1.findBindings)(currentPath);
        return { value: result.value };
    }
    if (!exec)
        return { error: new Error('EXEC_REQUIRED') };
    let content = '';
    try {
        const callbackName = '__taddy__';
        content = (0, buildCodeByPath_1.buildCodeByPath)(currentPath)
            // hack to avoid processing by babel-macro
            .replace(macroRe, config_1.PACKAGE_NAME)
            .concat(`\n\n;${callbackName}(${currentPath.toString()})`);
        // TODO: improve Hub type
        const { opts } = currentPath.hub.file;
        const { value, error } = evaluateChunk({
            content,
            filename: opts.filename,
            callbackName,
        });
        if (error)
            return { error };
        return { value: JSON.parse(value) };
    }
    catch (error) {
        // console.log('evaluate error', {content, error});
        return { error };
    }
}
exports.evaluate = evaluate;
//# sourceMappingURL=evaluate.js.map