"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.optimizeStaticStyles = void 0;
const tslib_1 = require("tslib");
const t = tslib_1.__importStar(require("@babel/types"));
const core_1 = require("@taddy/core");
const getObjectPropertyKey_1 = require("./getObjectPropertyKey");
function apply(quasis) {
    let expr;
    let curr = '';
    for (const v of quasis) {
        if (typeof v === 'string') {
            curr += (curr ? ' ' : '') + v;
            continue;
        }
        if (expr && curr) {
            expr = t.binaryExpression('+', expr, t.stringLiteral(' ' + curr + ' '));
            expr = t.binaryExpression('+', expr, v);
        }
        else if (curr) {
            expr = t.binaryExpression('+', t.stringLiteral(curr + ' '), v);
        }
        else {
            expr = v;
        }
        curr = '';
    }
    if (expr && curr) {
        expr = t.binaryExpression('+', expr, t.stringLiteral(' ' + curr));
    }
    return expr || t.stringLiteral(curr);
}
function optimizeStaticStyles(path) {
    // const quasis: string[] = [];
    // const expressions: any[] = [];
    const props = [];
    let quasis = [];
    for (const propPath of path.get('properties')) {
        if (!propPath.isObjectProperty()) {
            throw new Error('Cant optimize this path');
        }
        const keyPath = propPath.get('key');
        const key = (0, getObjectPropertyKey_1.getObjectPropertyKey)(keyPath);
        if (!key)
            return;
        const valuePath = propPath.get('value');
        if (key === 'className' || key === 'style' || key === core_1.VARS_KEY) {
            if (quasis.length) {
                props.push(t.objectProperty(apply(quasis), t.booleanLiteral(true)));
            }
            props.push(propPath.node);
            quasis = [];
            continue;
        }
        // if (key === 'className') {
        //     quasis.push(valuePath.node);
        //     continue;
        // }
        if (valuePath.isStringLiteral()) {
            quasis.push(key + valuePath.node.value);
            continue;
        }
        if (valuePath.isBooleanLiteral()) {
            quasis.push(key);
            continue;
        }
        quasis.push(t.binaryExpression('+', t.stringLiteral(key), valuePath.node));
        // expressions.push(
        //     t.binaryExpression(
        //         '+',
        //         t.stringLiteral(key),
        //         valuePath.node as t.Expression,
        //     ),
        // );
    }
    if (props.length === 0) {
        path.replaceWith(apply(quasis));
        return;
    }
    if (quasis.length) {
        props.push(t.objectProperty(apply(quasis), t.booleanLiteral(true)));
    }
    path.node.properties = props;
}
exports.optimizeStaticStyles = optimizeStaticStyles;
//# sourceMappingURL=optimizeStaticStyles.js.map