"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.taggedTemplateToObject = void 0;
const tslib_1 = require("tslib");
const t = tslib_1.__importStar(require("@babel/types"));
function taggedTemplateToObject(path) {
    const postcss = require('postcss');
    const postcssJS = require('postcss-js');
    const { quasis, expressions } = path.node.quasi;
    const cache = {};
    // this tricky placeholder has special syntax to avoid any conflicts with css parsing
    const re = /@\^var__\w+__/;
    const getPlaceholder = (index) => `@^var__${index}__`;
    let pseudoCSS = '';
    for (let i = 0; i < quasis.length; i++) {
        pseudoCSS += quasis[i].value.raw;
        if (expressions[i]) {
            const placeholder = getPlaceholder(i);
            cache[placeholder] = expressions[i];
            pseudoCSS += placeholder;
        }
    }
    const root = postcss.parse(pseudoCSS);
    const CSSObject = postcssJS.objectify(root);
    function parseString(str) {
        const expr = [];
        str.replace(new RegExp(re, 'g'), (match) => {
            expr.push(cache[match]);
        });
        if (expr.length === 0) {
            return t.stringLiteral(str);
        }
        return t.templateLiteral(str.split(re).map((q) => t.templateElement({ raw: q, cooked: q })), expr);
    }
    function traverse(obj) {
        const props = [];
        for (const key in obj) {
            const value = obj[key];
            if (value === true) {
                props.push(t.spreadElement(cache[key]));
                continue;
            }
            const cssKey = key.replace(/^&/, '');
            const keyNode = parseString(cssKey);
            let valueNode;
            if (typeof value === 'object') {
                valueNode = traverse(value);
            }
            else {
                valueNode = parseString(String(value));
            }
            props.push(t.objectProperty(keyNode, valueNode, !t.isStringLiteral(keyNode)));
        }
        return t.objectExpression(props);
    }
    return traverse(CSSObject);
}
exports.taggedTemplateToObject = taggedTemplateToObject;
//# sourceMappingURL=taggedTemplateToObject.js.map