import type { NodePath, PluginPass, ConfigAPI } from '@babel/core';
import { getEnv } from './config';
import { type OutputOptions } from './Output';
import type { ProcessorConfig } from './Processor';
import type { Target } from './types';
type CompileOptions = {
    /**
     * Apply static evaluation optimizations
     *
     * @default true
     */
    evaluate: ProcessorConfig['evaluate'];
    /**
     * Use typescript language server
     *
     * By default, true for typescript files
     * @default false
     */
    unstable_typescript: ProcessorConfig['typescript'];
    /**
     * Use CSS Variables fallback
     * If true, then even not optimized dynamic values would be optimized as possible
     *
     * @default true
     */
    unstable_CSSVariableFallback: ProcessorConfig['CSSVariableFallback'];
    /**
     * Based on static analysis remove unused bindings after compilation
     *
     * @default true
     */
    unstable_optimizeBindings: ProcessorConfig['optimizeBindings'];
    /**
     * Support tagged template literals.
     *
     * That's an experimental option that might be deprecated.
     *
     * @default false;
     */
    unstable_useTaggedTemplateLiterals: boolean;
    /**
     * Use sourcemaps
     *
     * @default true for development;
     */
    unstable_sourcemaps: boolean;
    /**
     * Set target
     * @default 'auto'
     */
    unstable_target: Target;
};
export type MacroConfig = Partial<{
    env?: ReturnType<typeof getEnv>;
    compileOptions: Partial<CompileOptions>;
    outputOptions: Partial<OutputOptions>;
}>;
export type MacroOptions = {
    state: PluginPass;
    babel: ConfigAPI;
    config?: MacroConfig;
    references: Record<string, NodePath[]>;
};
export declare function macro({ references, babel, state, config: _config, }: MacroOptions): {
    keepImports: boolean;
} | undefined;
export {};
