"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.macro = void 0;
const tslib_1 = require("tslib");
const t = tslib_1.__importStar(require("@babel/types"));
const assert_1 = tslib_1.__importDefault(require("assert"));
const path_1 = tslib_1.__importDefault(require("path"));
const taddy_1 = require("taddy");
const config_1 = require("./config");
const utils = tslib_1.__importStar(require("./helpers/utils.cjs"));
const taggedTemplateToObject_1 = require("./helpers/taggedTemplateToObject");
const handlers_1 = require("./handlers");
const Output_1 = require("./Output");
const source_maps_1 = require("./source-maps");
function mapCompileOptions({ filename, evaluate = true, unstable_typescript = false, unstable_CSSVariableFallback = true, unstable_optimizeBindings = true, unstable_useTaggedTemplateLiterals = false, unstable_target = 'auto', }) {
    const getTarget = () => {
        if (unstable_target !== 'auto')
            return unstable_target;
        const extname = path_1.default.extname(filename);
        if (extname === '.vue')
            return 'vue';
        return 'auto';
    };
    return {
        evaluate,
        typescript: unstable_typescript,
        CSSVariableFallback: unstable_CSSVariableFallback,
        optimizeBindings: unstable_optimizeBindings,
        useTaggedTemplateLiterals: unstable_useTaggedTemplateLiterals,
        target: getTarget(),
    };
}
const findImportPath = (program) => {
    let importPath = undefined;
    program.traverse({
        ImportDeclaration(p) {
            const { node } = p;
            if (!(node.source.value === config_1.PACKAGE_NAME ||
                node.source.value === config_1.MACRO_NAME)) {
                return;
            }
            importPath = p;
            importPath.node.source = t.stringLiteral('taddy');
            p.stop();
        },
    });
    return importPath;
};
let output;
let sourceMapGenerator;
function macro({ references, babel, state, config: _config = {}, }) {
    const { env = (0, config_1.getEnv)(babel), ...config } = _config;
    if (utils.isTaddyEvaluation(state)) {
        return;
    }
    const program = state.file.path;
    const code = state.file.code;
    const { filename, sourceRoot, root, filenameRelative = path_1.default.relative(sourceRoot || root || process.cwd(), filename), } = state.file.opts;
    (0, assert_1.default)(filename, 'Filename is required');
    (0, assert_1.default)(filenameRelative, 'FilenameRelative is required');
    const importPath = findImportPath(program);
    if (!importPath)
        return;
    sourceMapGenerator = (0, source_maps_1.makeSourceMapGenerator)(state.file);
    sourceMapGenerator.setSourceContent(filename, code);
    // TODO: implement "createCSS" and pass it
    taddy_1.$css.ruleInjector.reset();
    const importCache = new Map();
    const { useTaggedTemplateLiterals, ...compileOptions } = mapCompileOptions({
        filename,
        ...config.compileOptions,
    });
    switch (compileOptions.target) {
        case 'vue': {
            require('taddy/vue');
            break;
        }
        case 'auto': {
            break;
        }
    }
    const { handlers, finish } = (0, handlers_1.createHandlers)(compileOptions, {
        state,
        sourceMapGenerator,
        env,
        filename,
        filenameRelative,
        code,
        addImport(name) {
            if (!importCache.has(name)) {
                const specifier = t.importSpecifier(t.identifier(`_${name}_`), t.identifier(name));
                importCache.set(name, specifier);
                importPath.node.specifiers.push(specifier);
            }
            return importCache.get(name).local;
        },
    });
    for (const key in references) {
        if (!(key in handlers))
            continue;
        for (const ref of references[key]) {
            let currentPath = ref.parentPath;
            let currentKey = key;
            if (key === 'css' &&
                currentPath?.isMemberExpression() &&
                'name' in currentPath.node.property &&
                currentPath.node.property.name === 'mixin') {
                currentPath = currentPath.parentPath;
                currentKey = 'mixin';
            }
            if (useTaggedTemplateLiterals &&
                (currentKey === 'css' || currentKey === 'mixin') &&
                currentPath?.isTaggedTemplateExpression()) {
                const obj = (0, taggedTemplateToObject_1.taggedTemplateToObject)(currentPath);
                currentPath.replaceWith(t.callExpression(currentPath.node.tag, [obj]));
            }
            handlers[currentKey](currentPath);
        }
    }
    const { isStatic } = finish();
    if (isStatic) {
        importPath.node.source = t.stringLiteral('@taddy/core');
    }
    output = output || new Output_1.Output({ env, config: config.outputOptions });
    const sourceMap = (0, source_maps_1.convertGeneratorToComment)(sourceMapGenerator);
    const result = output.save({
        sourceMap,
        filenameRelative,
        target: compileOptions.target,
    });
    importPath.insertAfter(t.importDeclaration([], t.stringLiteral(result.importName)));
    return {
        keepImports: true,
    };
}
exports.macro = macro;
//# sourceMappingURL=macro-plugin.js.map