"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const config_1 = require("./config");
const utils = tslib_1.__importStar(require("./helpers/utils.cjs"));
const macro_plugin_1 = require("./macro-plugin");
function getImportNames(path) {
    const localName = path.node.local.name;
    if (path.isImportSpecifier()) {
        const imported = path.node.imported;
        return {
            localName,
            importedName: imported.name,
        };
    }
    if (path.isImportDefaultSpecifier()) {
        return { localName, importedName: 'default' };
    }
    return { localName, importedName: 'namespace' };
}
function findReferences(paths) {
    const references = {};
    for (const path of paths) {
        const { localName, importedName } = getImportNames(path);
        const binding = path.scope.getBinding(localName);
        if (binding?.referencePaths) {
            references[importedName] = binding.referencePaths;
        }
    }
    return references;
}
function plugin(babel, options) {
    const env = (0, config_1.getEnv)(babel);
    return {
        name: '@taddy/babel-plugin',
        pre() {
            this.references = {};
        },
        visitor: {
            // TODO: support require expression
            ImportDeclaration(path, state) {
                // console.log('run', state.file.code, state.filename);
                if (utils.isTaddyEvaluation(state)) {
                    return;
                }
                if (path.node.source.value !== config_1.PACKAGE_NAME) {
                    return;
                }
                Object.assign(this.references, findReferences(path.get('specifiers')));
                if (!(this.references.css || this.references.mixin)) {
                    return;
                }
                (0, macro_plugin_1.macro)({
                    references: this.references,
                    babel,
                    state,
                    config: {
                        env,
                        ...options,
                    },
                });
            },
        },
    };
}
exports.default = plugin;
//# sourceMappingURL=plugin.js.map